@@ -3,9 +3,9 @@
  *
  * TOMOYO Linux's utilities.
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 2.3.0   2010/08/20
+ * Version: 2.3.0+   2011/09/29
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -746,7 +746,7 @@ static void tomoyo_check_number_group_policy(char *data)
 	data = cp;
 	cp = strchr(data, '-');
 	if (cp)
-		*cp = '\0';
+		*cp++ = '\0';
 	if (!tomoyo_parse_ulong(&v, &data) || *data) {
 		printf("%u: ERROR: '%s' is a bad number.\n", tomoyo_line, data);
 		tomoyo_errors++;
